/* This file was generated by upbc (the upb compiler) from the input
 * file:
 *
 *     envoy/service/discovery/v3/ads.proto
 *
 * Do not edit -- your changes will be discarded when the file is
 * regenerated. */

#ifndef ENVOY_SERVICE_DISCOVERY_V3_ADS_PROTO_UPB_H_
#define ENVOY_SERVICE_DISCOVERY_V3_ADS_PROTO_UPB_H_

#include "upb/msg_internal.h"
#include "upb/decode.h"
#include "upb/decode_fast.h"
#include "upb/encode.h"

#include "upb/port_def.inc"

#ifdef __cplusplus
extern "C" {
#endif

struct envoy_service_discovery_v3_AdsDummy;
typedef struct envoy_service_discovery_v3_AdsDummy envoy_service_discovery_v3_AdsDummy;
extern const upb_MiniTable envoy_service_discovery_v3_AdsDummy_msginit;



/* envoy.service.discovery.v3.AdsDummy */

UPB_INLINE envoy_service_discovery_v3_AdsDummy* envoy_service_discovery_v3_AdsDummy_new(upb_Arena* arena) {
  return (envoy_service_discovery_v3_AdsDummy*)_upb_Message_New(&envoy_service_discovery_v3_AdsDummy_msginit, arena);
}
UPB_INLINE envoy_service_discovery_v3_AdsDummy* envoy_service_discovery_v3_AdsDummy_parse(const char* buf, size_t size, upb_Arena* arena) {
  envoy_service_discovery_v3_AdsDummy* ret = envoy_service_discovery_v3_AdsDummy_new(arena);
  if (!ret) return NULL;
  if (upb_Decode(buf, size, ret, &envoy_service_discovery_v3_AdsDummy_msginit, NULL, 0, arena) != kUpb_DecodeStatus_Ok) {
    return NULL;
  }
  return ret;
}
UPB_INLINE envoy_service_discovery_v3_AdsDummy* envoy_service_discovery_v3_AdsDummy_parse_ex(const char* buf, size_t size,
                           const upb_ExtensionRegistry* extreg,
                           int options, upb_Arena* arena) {
  envoy_service_discovery_v3_AdsDummy* ret = envoy_service_discovery_v3_AdsDummy_new(arena);
  if (!ret) return NULL;
  if (upb_Decode(buf, size, ret, &envoy_service_discovery_v3_AdsDummy_msginit, extreg, options, arena) !=
      kUpb_DecodeStatus_Ok) {
    return NULL;
  }
  return ret;
}
UPB_INLINE char* envoy_service_discovery_v3_AdsDummy_serialize(const envoy_service_discovery_v3_AdsDummy* msg, upb_Arena* arena, size_t* len) {
  return upb_Encode(msg, &envoy_service_discovery_v3_AdsDummy_msginit, 0, arena, len);
}
UPB_INLINE char* envoy_service_discovery_v3_AdsDummy_serialize_ex(const envoy_service_discovery_v3_AdsDummy* msg, int options,
                                 upb_Arena* arena, size_t* len) {
  return upb_Encode(msg, &envoy_service_discovery_v3_AdsDummy_msginit, options, arena, len);
}


extern const upb_MiniTable_File envoy_service_discovery_v3_ads_proto_upb_file_layout;

#ifdef __cplusplus
}  /* extern "C" */
#endif

#include "upb/port_undef.inc"

#endif  /* ENVOY_SERVICE_DISCOVERY_V3_ADS_PROTO_UPB_H_ */
