/* This file was generated by upbc (the upb compiler) from the input
 * file:
 *
 *     xds/annotations/v3/sensitive.proto
 *
 * Do not edit -- your changes will be discarded when the file is
 * regenerated. */

#include <stddef.h>
#include "upb/msg_internal.h"
#include "xds/annotations/v3/sensitive.upb.h"
#include "google/protobuf/descriptor.upb.h"

#include "upb/port_def.inc"

extern const upb_MiniTable google_protobuf_FieldOptions_msginit;
const upb_MiniTable_Extension xds_annotations_v3_sensitive_ext = {
  {61008053, UPB_SIZE(0, 0), UPB_SIZE(0, 0), kUpb_NoSub, 8, kUpb_FieldMode_Scalar | kUpb_LabelFlags_IsExtension | (kUpb_FieldRep_1Byte << kUpb_FieldRep_Shift)},
  &google_protobuf_FieldOptions_msginit,
  {.submsg = NULL},

};

static const upb_MiniTable_Extension *extensions_layout[1] = {
  &xds_annotations_v3_sensitive_ext,
};

const upb_MiniTable_File xds_annotations_v3_sensitive_proto_upb_file_layout = {
  NULL,
  NULL,
  extensions_layout,
  0,
  0,
  1,
};

#include "upb/port_undef.inc"

